% Mandible Surface Data and Tools
%
% (C) 2013 Moo K. Chung, Houri K. Vorperian
%Vocal Tract Development Laboratory 
%Waisman Center 
% University of Wisconsin-Madison
% mkchung@wisc.edu

%------------------------------------------
% MANDIBLE GROWTH ANALYSIS

load mandible77subjects.mat   

% template surface
template.faces=faces;
template.vertices=voxelsize(44)*mean(vertices,3);
figure; figure_patch(template,[0.74 0.71 0.61],0.7);
view([90 60]); camlight;
title('Template')

print_tif('template')

%-----------------------------------
% Extracting gender and age info from id

str=cell2mat(id);
group=str(:,1)
group=[group=='M']  % M=1, F=0
year = str2num(str(:,6:7));
month=str2num(str(:,9:10));
age = (12*year+month)/12;

%-----------------------------------------
% 21639 vertices are subsampled to speed up computation 

tt=reducepatch(template,0.1);
ind=mesh_commonvertex(template, tt);
p= vertices(ind,:,:);

%-------------
% age bins
age1= age<7;
age2=(7<=age)&(age<13);
age3=13<=age;

%---------------
% displacement lengths in each age bins

p1= p(:,:,age1);
p2= p(:,:,age2);
p3= p(:,:,age3);

p1L=squeeze(sqrt(sum(p1.^2,2)));
p2L=squeeze(sqrt(sum(p2.^2,2)));
p3L=squeeze(sqrt(sum(p3.^2,2)));

p12=mean(p2,3)-mean(p1,3);
p23=mean(p3,3)-mean(p2,3);

p12L=sqrt(sum(p12.^2,2));
p23L=sqrt(sum(p23.^2,2));

%-----------------------
% visualizing age effects 
s=voxelsize(44);

figure; figure_surf(tt, s*p12L);
figure_quiver3surface(tt, s*5*p12, 5);
caxis([0 10*s]);
colorbar
title('Between age group I and II')
colormap('hot')
view([90 60]); 
camlight;

figure; figure_surf(tt, s*p23L);
figure_quiver3surface(tt, s*5*p23, 5);
caxis([0 10*s]);
colorbar
title('Between age group II and III')
colormap('hot'); 
view([90 60]); 
camlight; 


%-----------------------------------------------------
%Obtain LB-eigenfunctions
% eigenvalues D is sorted in increasing order. 
% V is the eigenfunctions of the LB-operator.

[A, C]= FEM(tt);
[V, D]= eigs(C,A, 1000,'sa');  

% Displaying 5th eigenfunction
figure; figure_surf(tt,V(:,5))
view([90 60]); camlight;

%----------------------------------------
% Heat kernel smoothing on displacement lengths

p1Lsmooth=lb_smooth(p1L,tt, 20, 200, V, D);
p2Lsmooth=lb_smooth(p2L,tt, 20, 200, V, D);
p3Lsmooth=lb_smooth(p3L,tt, 20, 200, V, D);

%-------------------------
% Two sample T-test on the length difference
% Note T^2 = F test.

tfield=[];
for i=1:length(p1Lsmooth)
[h p, ci, t] = ttest2(p2Lsmooth(i,:), p1Lsmooth(i,:),[],[],'equal');
tfield=[tfield t.tstat];
end;

s=voxelsize(44); %voxel size
figure; figure_surf(tt, tfield);
figure_quiver3surface(tt, s*5*p12, 4);
view([90 60]); colormap('hot'); colorbar
title('T-stat between age group I and II')
caxis([-6 6]); colorbar; camlight

tfield=[];
for i=1:length(p1Lsmooth)
[h p, ci, t] = ttest2(p3Lsmooth(i,:), p2Lsmooth(i,:), [],[], 'equal');
tfield=[tfield t.tstat];
end;

figure; figure_surf(tt, tfield);
figure_quiver3surface(tt, s*5*p12, 4);
view([90 60]); colormap('hot'); colorbar
title('T-stat between age group II and III')
caxis([-6 6]); colorbar; camlight


%----------------------------------------------
% Hotelling's T2 result
% The resulting F-stat is the square of T-stat. 

tfield=[];
for i=1:length(p1Lsmooth)
    h= hotelT2(p2Lsmooth(i,:)', p1Lsmooth(i,:)');
    tfield=[tfield  h.t];
end;

figure; figure_surf(tt, tfield);
figure_quiver3surface(tt, s*5*p12, 4);
view([90 60]); 

caxis([0 16]); colorbar
camlight; colormap('hot')

c=colormap;
colormap(c(end:-1:1,:));
title('Hotelling T-square between age group I and II')


%----------------------------------
% RANDOM FIELD THEORY: corrected p-value for F-map

t=[9 12 16]
a=1; b=sum(age1) + sum(age2) -2;
sigma=20;

mu0=2;

% surface has to be scaled properly by the voxel size
ss.faces = tt.faces;
ss.vertices= s*tt.vertices;
area=GETarea(ss);
mu2=sum(area.faces)/2;

rho0 = 1 - fcdf(t,a,b);

rho2 = 1/(4*sigma^2*pi)*gamma((a+b-2)/2)/ gamma(a/2)/gamma(b/2)...
    *(a*t/b).^((a-2)/2).*(1+a*t/b).^(-(a+b-2)/2).*((b-1)*a*t/b-(a-1));

p=mu0*rho0+rho2*mu2

% F-stat
F.tstat=[9 12 16]
F.df= [a b];
stat=stat_Fcorrected(F, sigma,ss)


%----------------------
% p-value plot and thresholding

F.tstat=[8:0.01:20]
a=1; b=sum(age1) + sum(age2) -2;
F.df= [a b];

stat=stat_Fcorrected(F,20,ss)
figure; plot(F.tstat, stat.pcorrected, '-k', 'LIneWidth', 2)
title('Corrected p-value')
xlabel('F-statistic')
ylabel('p-value')

ind=find(stat.pcorrected<0.01);
F.tstat(ind(1))


%-------------------------------------------------
% Statstical power: polynomial regression 

power=zeros(2165,1);
X=[ones(77,1) group age] % age.*group age.^2 age.^3 age.^4];
N= 77 %820 %97;
pn=3; %number of parameters on ther right including intercept
alpha=0.01;

q=squeeze(sqrt(sum(p.^2,2)));


for i=1:2165
    y=q(i,:);
    
    [b,bint, R2,rint,stats] = regress(y', X);
    
    SSE= sum((y'-X*b).^2);
    SST=sum((y'-mean(y')).^2);
    R2= (SST-SSE)/SSE;
    
    
    f2 = R2/(1-R2);
    lambda= N*f2; % noncentrality
    power(i) = 1- ncfcdf( finv(1-alpha, pn-1, N-pn), pn-1, N-pn, lambda);
 
end;

figure; figure_surf(tt, power); colorbar

%----------------------------------------------
% Statisticl power for two sample t-test


for i=0:2:18
agea=12
age1= ( agea<=age)&(age<agea+1);
age2=(agea+1<=age)&(age<agea+2);

n1=sum(age1);
n2=sum(age2);

p1= p(:,:,age1);
p2= p(:,:,age2);

p1L=squeeze(sqrt(sum(p1.^2,2)));
p2L=squeeze(sqrt(sum(p2.^2,2)));

% mean.
p12=mean(p2L,2)-mean(p1L,2);

std12 = sqrt(((n1-1)*var(p1L') + (n2-1)*var(p2L'))/(n1+n2-2));

c= abs(p12)./std12';

power1 = inline('1-normcdf(2.33 - c./sqrt(2./n))','n','c');

n=(n1+n2)/2*820/77;
power=power1(n,c);

figure; figure_surf(tt, power); colormap('hot'); colorbar; caxis([0 1]);
view([90 60]); camlight;
title( strcat('Statistical power between ages   ', int2str(agea), '  and  ', int2str(agea+2)));

filename=strcat('power', int2str(i))
print_tif(filename)

end;


% point 

x=[17.28 11.92 97.27]  %A

x=[84.99 62.49 26.46]  %B

x=[69.18 66.53  30.86] %C

diff= tt.vertices - repmat(x, 2165,1)
diff=L2norm(diff)
ind = find(diff==min(diff))



agea=12
age1= ( agea<=age)&(age<agea+1);
age2=(agea+1<=age)&(age<agea+2);

n1=sum(age1);
n2=sum(age2);

p1= p(:,:,age1);
p2= p(:,:,age2);

p1L=squeeze(sqrt(sum(p1.^2,2)));
p2L=squeeze(sqrt(sum(p2.^2,2)));

% mean.
p12=mean(p2L,2)-mean(p1L,2);

std12 = sqrt(((n1-1)*var(p1L') + (n2-1)*var(p2L'))/(n1+n2-2));

c= abs(p12)./std12';

ceff=c(ind)

power1 = inline('1-normcdf(2.33 - c./sqrt(2./n))','n','c');

n=77:820;
nnorm=(n1+n2)/2*n/77;


hold on; plot(n, power1(nnorm, ceff), '-k', 'LineWidth', 2)
axis([77 820 0 1]); box on
figure_bg('w')
xlabel('n')
ylabel('power')

print_tif('powerplot-mandible')




